'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _security = require('./security');

var _security2 = _interopRequireDefault(_security);

var _q = require('q');

var _q2 = _interopRequireDefault(_q);

var _models = require('./models/');

var _models2 = _interopRequireDefault(_models);

var _sequelize = require('sequelize');

var _sequelize2 = _interopRequireDefault(_sequelize);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * La clase DcRbac encapsula los metodos necesario
 * para poder trabajar con el modelo RBAC definido
 * en la base de datos.
 *  
 */
var DcRbac = function () {
  /**
   * Crea una nueva instancia de DcRbac con opciones (ops).
   *
   * Opciones:
   *  dbConfig // db opciones
   *    port     // puerto de conexión
   *    database // nombre de la base de datos
   *    user     // nombre de usuario para conexión
   *    pwd      // contraseña de conexión
   *  logging  // habilita logs 
   *  
   * @param  {Object} ops Opciones
   * @return {Object}     Nueva instancia de la clase
   */
  function DcRbac(ops) {
    _classCallCheck(this, DcRbac);

    ops = ops || {};
    var db = ops.dbConfig || {};
    db.database = 'rbac';
    db.user = 'postgres';
    db.port = db.port || 5432;
    db.pwd = db.pwd || '';
    var logging = ops.logging || false;

    this.secret = ops.secret || '0198273498327465';

    this.security = new _security2.default({ secret: this.secret });

    this.sequelize = new _sequelize2.default(db.database, db.user, db.pwd, {
      dialect: "postgres",
      port: db.port,
      timestamps: false,
      logging: logging
    });

    this.models = new _models2.default(this.sequelize, _sequelize2.default);
  }

  /**
   * Agrega un nuevo perfil.
   *
   * Veasé Modelo RBAC [profiles]
   * 
   * @param {Object} profile Objecto con información del perfil
   * @promise {Object}     Q.promise
   */


  _createClass(DcRbac, [{
    key: 'addProfile',
    value: function addProfile(profile) {
      var _this = this;

      var def = _q2.default.defer();

      var profile_name = profile.profile_name;

      var dbProfile = {
        profile_name: profile_name
      };

      this.sequelize.transaction(function (t) {
        return _this.models.profiles.create(dbProfile).then(function (p) {
          def.resolve(p);
        }).catch(function (err) {
          def.reject(err);
        });
      });

      return def.promise;
    }

    /**
     * Agrega un nuevo role.
     *
     * Veasé Modelo RBAC [roles]
     * 
     * @param {Object} role Objecto con información del role
     * @promise {Object}     Q.promise
     */

  }, {
    key: 'addRole',
    value: function addRole(role) {
      var _this2 = this;

      var def = _q2.default.defer();

      var role_name = role.role_name;

      var dbRole = {
        role_name: role_name
      };

      this.sequelize.transaction(function (t) {
        return _this2.models.roles.create(dbRole).then(function (p) {
          def.resolve(p);
        }).catch(function (err) {
          def.reject(err);
        });
      });

      return def.promise;
    }

    /**
     * Agrega una nueva applicación.
     *
     * Veasé Modelo RBAC [applications]
     * 
     * @param {Object} object Objecto con información de la application
     * @promise {Object}     Q.promise
     */

  }, {
    key: 'addApplication',
    value: function addApplication(obj) {
      var _this3 = this;

      var def = _q2.default.defer();

      var application_name = obj.application_name;

      var dbObj = {
        application_name: application_name
      };

      this.sequelize.transaction(function (t) {
        return _this3.models.applications.create(dbObj).then(function (o) {
          def.resolve(o);
        }).catch(function (err) {
          def.reject(err);
        });
      });

      return def.promise;
    }

    /**
     * Agrega un nuevo object de aplicación.
     *
     * Veasé Modelo RBAC [objects]
     * 
     * @param {Object} object Objecto con información del object
     * @promise {Object}     Q.promise
     */

  }, {
    key: 'addObject',
    value: function addObject(obj) {
      var _this4 = this;

      var def = _q2.default.defer();

      var object_name = obj.object_name;
      var application_id_applications = obj.application_id_applications;

      var dbObj = {
        object_name: object_name,
        application_id_applications: application_id_applications
      };

      this.sequelize.transaction(function (t) {
        return _this4.models.objects.create(dbObj).then(function (o) {
          def.resolve(o);
        }).catch(function (err) {
          def.reject(err);
        });
      });

      return def.promise;
    }

    /**
     * Agrega un token.
     *
     * Veasé Modelo RBAC [token]
     * 
     * @param {Object} object Objecto con información del token
     * @promise {Object}     Q.promise
     */

  }, {
    key: 'createToken',
    value: function createToken(obj) {
      var _this5 = this;

      var def = _q2.default.defer();

      var user_id_users = obj.user_id_users;
      var code = this.security.randomCode();
      var days = days === undefined ? 1 : obj.days;
      var token_salt = this.security.radomSalt();

      var creation = new Date();
      var expiration = (0, _moment2.default)(creation).add(days, 'days');

      var token = this.security.encrypt(code);
      var type = obj.type || 'activation';

      var dbObj = {
        token: token,
        user_id_users: user_id_users,
        type: type,
        expiration: expiration,
        token_salt: token_salt
      };

      this.sequelize.transaction(function (t) {
        return _this5.models.tokens.create(dbObj).then(function (t) {
          def.resolve({ token: t.dataValues, code: code });
        }).catch(function (err) {
          def.reject(err);
        });
      });

      return def.promise;
    }
  }, {
    key: 'getTokenByHash',
    value: function getTokenByHash(hash) {
      var def = _q2.default.defer();

      this.models.tokens.findOne({ where: { token: hash } }).then(function (token) {
        if (!token) {
          return def.resolve(null);
        }
        def.resolve(token.dataValues);
      }).catch(function (err) {
        def.reject(err);
      });

      return def.promise;
    }

    /**
     * Agrega un nuevo usuario.
     *
     * Veasé Modelo RBAC [users]
     * 
     * @param  {Object} user Objecto con información del user
     * @promise {Object}     Q.promise
     */

  }, {
    key: 'createUser',
    value: function createUser(user) {
      var _this6 = this;

      var def = _q2.default.defer();

      var pid = user.profile_id_profiles;
      var username = user.username;
      var email = user.email;
      var password = user.password;
      var signon_type = user.signon_type || 'local';
      var first_name = user.first_name || '';
      var last_name = user.last_name || '';
      var user_state = user.user_state || 'verifying';
      var pwdObj = this.security.generatePassword(user.password);

      var google_id = user.google_id;
      var account_image = user.account_image;
      var account_google_url = user.account_google_url;

      var salt = pwdObj.salt;
      var pwd = pwdObj.hash;

      var dbUser = {
        //user_id
        profile_id_profiles: user.profile_id_profiles,
        first_name: first_name,
        last_name: last_name,
        username: user.username,
        email: user.email,
        password: pwd,
        signon_type: signon_type,
        user_salt: salt,
        user_state: user_state,
        google_id: user.google_id,
        account_image: user.account_image,
        account_google_url: user.account_google_url
      };

      this.sequelize.transaction(function (t) {
        return _this6.models.users.create(dbUser).then(function (u) {
          def.resolve(_this6.mapUserPublic(u.dataValues));
        }).catch(function (err) {
          def.reject(err);
        });
      });

      return def.promise;
    }

    /**
     * Cambia la contraseña y salt del usuario
     * 
     * @param  {Integer} userId 
     * @param  {String} newPassword 
     * @promise {Object}     Q.promise
     */

  }, {
    key: 'changeUserPassword',
    value: function changeUserPassword(userId, newPassword) {
      var _this7 = this;

      var def = _q2.default.defer();

      this.getUserById(userId).then(function (user) {
        if (!user) {
          return def.reject(new Error('Unknown user'));
        }

        var pwdObj = _this7.security.generatePassword(newPassword);

        var salt = pwdObj.salt;
        var pwd = pwdObj.hash;

        _this7.sequelize.transaction(function (t) {
          return _this7.models.users.update({ password: pwd, user_salt: salt }, { where: { user_id: userId } }).then(function (affectedRows) {
            def.resolve(affectedRows);
          }).catch(function (err) {
            def.reject(err);
          });
        });
      }).catch(function (err) {
        def.reject(err);
      });

      return def.promise;
    }

    /**
     * Habilita un usuario
     * 
     * @param  {Integer} userId 
     * @promise {Object}     Q.promise
     */

  }, {
    key: 'enableUser',
    value: function enableUser(userId) {
      var _this8 = this;

      var def = _q2.default.defer();

      this.getUserById(userId).then(function (user) {
        if (!user) {
          return def.reject(new Error('Unknown user'));
        }

        _this8.sequelize.transaction(function (t) {
          return _this8.models.users.update({ user_state: 'enabled' }, { where: { user_id: userId } }).then(function (affectedRows) {
            def.resolve(affectedRows);
          }).catch(function (err) {
            def.reject(err);
          });
        });
      }).catch(function (err) {
        def.reject(err);
      });

      return def.promise;
    }

    /**
     * Deshabilita un usuario
     * 
     * @param  {Integer} userId 
     * @promise {Object}     Q.promise
     */

  }, {
    key: 'disableUser',
    value: function disableUser(userId) {
      var _this9 = this;

      var def = _q2.default.defer();

      this.getUserById(userId).then(function (user) {
        if (!user) {
          return def.reject(new Error('Unknown user'));
        }

        _this9.sequelize.transaction(function (t) {
          return _this9.models.users.update({ user_state: 'disabled' }, { where: { user_id: userId } }).then(function (affectedRows) {
            def.resolve(affectedRows);
          }).catch(function (err) {
            def.reject(err);
          });
        });
      }).catch(function (err) {
        def.reject(err);
      });

      return def.promise;
    }

    /**
     * Actualiza el acceso para un usuario y un objecto.
     * 
     * @param  {Integer} user_id      
     * @param  {Integer} object_id    
     * @param  {Integer} access_grant 
     * @param  {Integer} access_deny  
     * @promise {Object} Q.promise
     */

  }, {
    key: 'revokeUserObject',
    value: function revokeUserObject(user_id, object_id, access_grant, access_deny) {
      return this.executeFn('fn_revoke_user_object', {
        user_id: user_id,
        object_id: object_id,
        access_grant: access_grant,
        access_deny: access_deny
      });
    }

    /**
     * Elimina la relación entre un usuario dado y un objeto.
     * 
     * @param  {[type]} user_id   
     * @param  {[type]} object_id 
     * @promise {Object}     Q.promise
     */

  }, {
    key: 'unlinkUserObject',
    value: function unlinkUserObject(user_id, object_id) {
      return this.executeFn('fn_unlink_user_object', {
        user_id: user_id,
        object_id: object_id
      });
    }

    /**
     * Actualiza (insert/update) el acceso para un role y un objecto.
     * 
     * @param  {Integer} role_id      
     * @param  {Integer} object_id    
     * @param  {Integer} access_grant 
     * @param  {Integer} access_deny  
     * @promise {Object} Q.promise
     */

  }, {
    key: 'revokeRoleObject',
    value: function revokeRoleObject(role_id, object_id, access_grant, access_deny) {
      return this.executeFn('fn_revoke_role_object', {
        role_id: role_id,
        object_id: object_id,
        access_grant: access_grant,
        access_deny: access_deny
      });
    }

    /**
     * Elimina la relación entre un role dado y un objeto.
     * 
     * @param  {Integer} role_id   
     * @param  {Integer} object_id 
     * @promise {Object} Q.promise
     */

  }, {
    key: 'unlinkRoleObject',
    value: function unlinkRoleObject(role_id, object_id) {
      return this.executeFn('fn_unlink_role_object', {
        role_id: role_id,
        object_id: object_id
      });
    }

    /**
     * Obtiene el tipo de acceso que posee un role para un objecto dado.
     * (OR access_grant) AND (OR access_deny)
     * 
     * @param  {Integer} role_id   
     * @param  {Integer} object_id 
     * @promise {Object} Q.promise
     */

  }, {
    key: 'accessRoleObject',
    value: function accessRoleObject(role_id, object_id) {
      return this.executeFn('fn_access_role_object', {
        role_id: role_id,
        object_id: object_id
      });
    }

    /**
     * Obtiene el tipo de acceso que posee un usuario para un objecto dado.
     * 
     * @param  {Integer} user_id   
     * @param  {Integer} object_id 
     * @promise {Object} Q.promise
     */

  }, {
    key: 'accessUserObject',
    value: function accessUserObject(user_id, object_id) {
      return this.executeFn('fn_access_user_object', {
        user_id: user_id,
        object_id: object_id
      });
    }

    /**
     * Obtiene el tipo de acceso que posee un perfil para un objecto dado.
     * 
     * @param  {Integer} profile_id   
     * @param  {Integer} object_id 
     * @promise {Object} Q.promise
     */

  }, {
    key: 'accessProfileObject',
    value: function accessProfileObject(profile_id, object_id) {
      return this.executeFn('fn_access_profile_object', {
        profile_id: profile_id,
        object_id: object_id
      });
    }

    /**
     * Relaciona un role a un perfile.
     * 
     * @param {Integer} role_id    
     * @param {Integer} profile_id 
     * @promise {Object} Q.promise
     */

  }, {
    key: 'addRoleProfile',
    value: function addRoleProfile(role_id, profile_id) {
      return this.executeFn('fn_add_role_profile', {
        role_id: role_id,
        profile_id: profile_id
      });
    }

    /**
    * Elimina la relación para role a un perfile.
    * 
    * @param {Integer} role_id    
    * @param {Integer} profile_id 
    * @promise {Object} Q.promise
    */

  }, {
    key: 'removeRoleProfile',
    value: function removeRoleProfile(role_id, profile_id) {
      return this.executeFn('fn_remove_role_profile', {
        role_id: role_id,
        profile_id: profile_id
      });
    }

    /**
     * Obtiene lista de usuario.
     * 
     * @promise {Object} Q.promise
     */

  }, {
    key: 'getUsers',
    value: function getUsers() {
      var _this10 = this;

      var query = 'SELECT * FROM public.v_users';
      var def = _q2.default.defer();

      this.executeSelect(query).then(function (res) {
        var result = res.map(function (u) {
          return _this10.mapUserPublic(u);
        });
        def.resolve(result);
      }).catch(function (err) {
        def.reject(err);
      });

      return def.promise;
    }

    /**
     * Obtiene un un usuario por id.
     * 
     * @param  {Integer} userId   
     * @param  {Boolean} populate Define si se debe obtener el perfil
     * @promise {Object} Q.promise
     */

  }, {
    key: 'getUserById',
    value: function getUserById(userId, populate) {
      var _this11 = this;

      var def = _q2.default.defer();
      this.models.users.findOne({ where: { user_id: userId } }).then(function (res) {
        if (!res || !res.dataValues) {
          def.resolve(null);
        }
        var u = _this11.mapUserPublic(res.dataValues);
        if (!populate) {
          def.resolve(u);
        } else {
          _this11.getProfileById(u.profile_id_profiles).then(function (p) {
            u.profile = p;
            def.resolve(u);
          }).catch(function (e) {
            def.reject(e);
          });
        }
      }).catch(function (err) {
        def.reject(err);
      });

      return def.promise;
    }

    /**
     * Obtiene un un usuario por googl_id.
     * 
     * @param  {Integer} googlId   
     * @param  {Boolean} populate Define si se debe obtener el perfil
     * @promise {Object} Q.promise
     */

  }, {
    key: 'getUserByGoogleId',
    value: function getUserByGoogleId(googlId, populate) {
      var _this12 = this;

      var def = _q2.default.defer();
      this.models.users.findOne({ where: { google_id: googlId } }).then(function (res) {
        if (!res || !res.dataValues) {
          def.resolve(null);
        }
        var u = _this12.mapUserPublic(res.dataValues);
        if (!populate) {
          def.resolve(u);
        } else {
          _this12.getProfileById(u.profile_id_profiles).then(function (p) {
            u.profile = p;
            def.resolve(u);
          }).catch(function (e) {
            def.reject(e);
          });
        }
      }).catch(function (err) {
        def.reject(err);
      });

      return def.promise;
    }

    /**
     * Obtiene un usuario por username o email
     * @param  {[type]} username username o email
     * @param  {Boolean} populate Define si se debe obtener el perfil
     * @promise {Object} Q.promise
     */

  }, {
    key: 'getUserByUsernameOrEmail',
    value: function getUserByUsernameOrEmail(username, populate) {
      var _this13 = this;

      var def = _q2.default.defer();
      this.models.users.findOne({ where: { $or: [{ username: username }, { email: username }] } }).then(function (res) {
        if (!res || !res.dataValues) {
          def.resolve(null);
        }
        var u = _this13.mapUserPublic(res.dataValues);
        if (!populate) {
          def.resolve(u);
        } else {
          _this13.getProfileById(u.profile_id_profiles).then(function (p) {
            u.profile = p;
            def.resolve(u);
          }).catch(function (e) {
            def.reject(e);
          });
        }
      }).catch(function (err) {
        def.reject(err);
      });

      return def.promise;
    }

    /**
     * Obtiene un profile por id
     * 
     * @param  {Integer} profileId 
     * @param  {Boolean} populate Define si se debe obtener lso roles
     * @promise {Object} Q.promise
     */

  }, {
    key: 'getProfileById',
    value: function getProfileById(profileId, populate) {
      var _this14 = this;

      var def = _q2.default.defer();
      this.models.profiles.findOne({ where: { profile_id: profileId } }).then(function (res) {
        if (!res || !res.dataValues) {
          def.resolve(null);
        }
        if (!populate) {
          def.resolve(res.dataValues);
        } else {
          var p = res.dataValues;
          _this14.getRolesForProfile(profileId).then(function (roles) {
            p.roles = roles;
            def.resolve(p);
          }).catch(function (e) {
            def.reject(e);
          });
        }
      }).catch(function (err) {
        def.reject(err);
      });

      return def.promise;
    }

    /**
     * Obtiene un profile por nombre
     * 
     * @param  {String} name 
     * @param  {Boolean} populate Define si se debe obtener lso roles
     * @promise {Object} Q.promise
     */

  }, {
    key: 'getProfileByName',
    value: function getProfileByName(name, populate) {
      var _this15 = this;

      var def = _q2.default.defer();
      this.models.profiles.findOne({ where: { profile_name: name } }).then(function (res) {
        if (!res || !res.dataValues) {
          def.resolve(null);
        }
        if (!populate) {
          def.resolve(res.dataValues);
        } else {
          var p = res.dataValues;
          _this15.getRolesForProfile(p.profile_id).then(function (roles) {
            p.roles = roles;
            def.resolve(p);
          }).catch(function (e) {
            def.reject(e);
          });
        }
      }).catch(function (err) {
        def.reject(err);
      });

      return def.promise;
    }

    /**
     * Obtiene los roles para un perfil por id
     * 
     * @param  {Integer} profileId
     * @promise {Object} Q.promise
     */

  }, {
    key: 'getRolesForProfile',
    value: function getRolesForProfile(profileId) {
      var query = 'SELECT * FROM public.v_profiles_roles WHERE profile_id = :profile_id';
      var def = _q2.default.defer();

      this.executeSelect(query, { profile_id: profileId }).then(function (res) {
        def.resolve(res);
      }).catch(function (err) {
        def.reject(err);
      });

      return def.promise;
    }

    /**
     * Obtiene todas la relaciones con objetos para un usuario
     * 
     * @param  {Integer} userId
     * @promise {Object} Q.promise
     */

  }, {
    key: 'getObjectsForUser',
    value: function getObjectsForUser(userId) {
      var query = 'SELECT * FROM public.v_user_objects WHERE user_id = :user_id';
      var def = _q2.default.defer();

      this.executeSelect(query, { user_id: userId }).then(function (res) {
        def.resolve(res);
      }).catch(function (err) {
        def.reject(err);
      });

      return def.promise;
    }

    /**
     * Obtiene todas la relaciones con objetos para un role
     * 
     * @param  {Integer} roleId
     * @promise {Object} Q.promise
     */

  }, {
    key: 'getObjectsForRole',
    value: function getObjectsForRole(roleId) {
      var query = 'SELECT * FROM public.v_roles_objects WHERE role_id = :role_id';
      var def = _q2.default.defer();

      this.executeSelect(query, { role_id: roleId }).then(function (res) {
        def.resolve(res);
      }).catch(function (err) {
        def.reject(err);
      });

      return def.promise;
    }
  }, {
    key: 'authenticate',
    value: function authenticate(user, pwd, cb) {
      var _this16 = this;

      this.models.users.findOne({ where: { user_id: user.user_id } }).then(function (res) {
        if (!res || !res.dataValues) {
          def.resolve(null);
        }
        var savedpwd = res.dataValues.password;
        var salt = res.dataValues.user_salt;

        try {
          var cp = _this16.security.comparePassword(pwd, savedpwd);

          return cb(null, cp);
        } catch (err) {
          return cb(err);
        }
      }).catch(function (err) {
        cb(err);
      });
    }

    //
    // PRIVATE
    // 

    /**
     * Mapea propiedades visibles de un usuario.
     * 
     * @param  {Object} u Usuario
     * @return {Object}   Usuario
     * @private
     */

  }, {
    key: 'mapUserPublic',
    value: function mapUserPublic(u) {
      delete u.password;
      delete u.user_salt;
      u.profile = { profile_id: u.profile_id_profiles };
      return u;
    }

    /**
     * Ejecuta una función de la base de datos.
     * 
     * @param  {String} fn       Nombre de la función
     * @param  {Object}   params Parametros
     * @promise {Object} Q.promise
     * @private
     */

  }, {
    key: 'executeFn',
    value: function executeFn(fn, params) {
      var def = _q2.default.defer();

      var replacements = Object.keys(params).map(function (p) {
        return ':' + p;
      });
      var rstr = replacements.join(',');

      this.sequelize.query('SELECT public.' + fn + '(' + rstr + ') as resul', {
        replacements: params,
        type: this.sequelize.QueryTypes.SELECT
      }).then(function (res) {
        var result = null;
        if (res.length > 0) {
          result = res[0].result;
        }
        def.resolve(result);
      }).catch(function (err) {
        def.reject(err);
      });

      return def.promise;
    }

    /**
     * Ejecuta una query en forma de select en la base de datos.
     * 
     * @param  {String} query Query
     * @param  {Object} params Parametros
     * @promise {Object} Q.promise
     * @private
     */

  }, {
    key: 'executeSelect',
    value: function executeSelect(query, replacements) {
      var def = _q2.default.defer();

      this.sequelize.query(query, {
        replacements: replacements,
        type: this.sequelize.QueryTypes.SELECT
      }).then(function (res) {
        def.resolve(res);
      }).catch(function (err) {
        def.reject(err);
      });

      return def.promise;
    }
  }]);

  return DcRbac;
}();

exports.default = DcRbac;